@@ -1,36 +1,34 @@
 package Proc::Queue;
 
 require 5.005;
-
-our $VERSION = '1.14_01';
-
 use strict;
 # use warnings;
 require Exporter;
 use Carp;
+
 use POSIX ":sys_wait_h";
 
 our @ISA = qw(Exporter);
+
 our %EXPORT_TAGS = ( all => [ qw( fork_now
-				  waitpids
-				  run_back
-				  run_back_now
-				  system_back
-				  system_back_now
-				  all_exit_ok
-				  running_now ) ] );
+				    waitpids
+				    run_back
+				    run_back_now
+				    system_back
+				    system_back_now
+				    all_exit_ok
+				    running_now ) ] );
+
 our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
 our @EXPORT = qw();
 
-# are we running on Windows?
-use constant _win => $^O=~/win32/i;
+our $VERSION = '1.11';
 
 # parameters
 my $queue_size=4; # max number of councurrent processes running.
 my $debug=0; # shows debug info.
-my $trace=0; # shows function calls.
+my $trace=0; # shows function enters.
 my $delay=0; # delay between fork calls.
-my $ignore_childs=0; # similar to $SIG{CHILD}='IGNORE';
 
 my $last=0; # last time fork was called.
 
@@ -46,13 +44,12 @@ my @captured;
 sub import {
   my ($pkg,@opts)=@_;
   my $i;
-  for ($i=0; $i<@opts; $i++) {
+  for ($i=0; $i<=$#opts; $i++) {
     my $o=$opts[$i];
     if( $o eq 'size'
         or $o eq 'debug'
         or $o eq 'trace'
-	or $o eq 'delay'
-	or $o eq 'ignore_childs') {
+	or $o eq 'delay' ) {
       $#opts>$i or croak "option '$o' needs a value";
       my $value=$opts[$i+1];
       { no strict qw( subs refs );
@@ -66,70 +63,63 @@ sub import {
 }
 
 sub delay {
-  return $delay unless @_;
+  my $d=shift;
   my $old_delay=$delay;
-  $delay=$_[0];
-  carp "Proc queue delay set to ${delay}s, it was $old_delay" if $debug;
+  if(defined $d) {
+    $delay=$d;
+    carp "Proc queue delay set to $delay sec., it was $old_delay" if $debug;
+  }
   return $old_delay;
 }
 
 sub size {
-  return $queue_size unless @_;
   my $size=shift;
   my $old_size=$queue_size;
-  croak "invalid value for Proc::Queue size ($size), min value is 1"
-    unless $size >= 1;
-  $queue_size=$size;
-  carp "Proc queue size set to $size, it was $old_size" if $debug;
+  if(defined $size) {
+    croak "invalid value for Proc::Queue size ($size), min value is 1"
+      unless $size >= 1;
+    $queue_size=$size;
+    carp "Proc queue size set to $queue_size, it was $old_size" if $debug;
+  }
   return $old_size;
 }
 
 sub debug {
-  return $debug unless @_;
+  my $d=shift;
   my $old_debug=$debug;
-  if ($_[0]) {
-    $debug=1;
-    carp "debug mode ON";
-  }
-  else {
-    $debug=0;
-    carp "debug mode OFF" if $old_debug;
+  if(defined $d) {
+    if ($d) {
+      $debug=1;
+      carp "Debug mode is now ON for Proc::Queue module";
+    }
+    else {
+      $debug=0;
+      carp "Debug mode is now OFF for Proc::Queue module" if $old_debug;
+    }
   }
   return $old_debug;
 }
 
 sub trace {
-  return $trace unless @_;
+  my $t=shift;
   my $old_trace=$trace;
-  if ($_[0]) {
-    $trace=1;
-    carp "trace mode ON" if $debug;
-  }
-  else {
-    $trace=0;
-    carp "trace mode OFF" if $debug;
+  if(defined $t) {
+    if ($t) {
+      $trace=1;
+      carp "Trace mode is now ON for Proc::Queue module" if $debug;
+    }
+    else {
+      $trace=0;
+      carp "Trace mode is now OFF for Proc::Queue module" if $debug;
+    }
   }
   return $old_trace;
 }
 
-sub ignore_childs {
-  return $ignore_childs unless @_;
-  my $old_ignore_childs=$ignore_childs;
-  if ($_[0]) {
-    $ignore_childs=1;
-    carp "ignore_childs mode ON" if $debug;
-  }
-  else {
-    $ignore_childs=0;
-    carp "ignore_childs mode OFF" if $debug;
-  }
-  return $old_ignore_childs;
-}
-
 # sub to store internally captured processes
 sub _push_captured {
-  push @captured, shift,$? unless $ignore_childs;
-  croak "captured stack is corrupted" if (@captured & 1)
+  push @captured, shift,$?;
+  croak "captured stack is corrupted" unless ($#captured & 1)
 }
 
 # do the real wait and housekeeping
@@ -158,7 +148,7 @@ sub new_wait () {
   if(@captured) {
     my $w=shift @captured;
     $?=shift @captured;
-    carp "Wait returning old child $w captured in wait" if $debug;
+    carp "Wait returning old child $w captured in fork" if $debug;
     return $w;
   }
   return _wait;
@@ -187,11 +177,7 @@ sub _waitpid ($$) {
 
 sub _clean() {
   my $pid;
-  while (1) {
-    $pid=_waitpid(-1,WNOHANG);
-    return unless ((_win && $pid < -1) || $pid >0);
-    _push_captured $pid
-  }
+  _push_captured($pid) while(($pid=_waitpid(-1,WNOHANG))>0);
 }
 
 sub new_waitpid ($$) {
@@ -222,7 +208,7 @@ BEGIN { eval "use Time::HiRes 'time'" }
 sub _fork () {
   carp "Proc::Queue::_fork called" if $trace && $debug;
   if ($delay>0) {
-    my $wait=$last+$delay - time;
+    my $wait=$last+$delay - time();
     if ($wait>0) {
       carp "Delaying $wait seconds before forking" if $debug;
       select (undef,undef,undef,$wait);
@@ -254,7 +240,7 @@ sub _fork () {
 sub new_fork () {
   carp "Proc::Queue::fork called" if $trace;
   while($queue_now>=$queue_size) {
-    carp "Waiting for some process to finish" if $debug;
+    carp "Waiting that some process finishes before continuing" if $debug;
     my $nw;
     if (($nw=_wait) != -1) {
       _push_captured $nw;
@@ -264,7 +250,7 @@ sub new_fork () {
       last;
     }
   }
-  return _fork;
+  return _fork();
 }
 
 sub fork_now () {
@@ -279,7 +265,7 @@ sub waitpids {
     if (defined($pid)) {
       carp "Waiting for child $pid to exit" if $debug;
       my $r=new_waitpid($pid,0);
-      if ((_win && $r < -1) || $r > 0) {
+      if ($r > 0) {
 	carp "Child $r return $?" if $debug;
 	push @result,$r,$?;
       }
@@ -357,10 +343,10 @@ sub all_exit_ok {
   carp "Proc::Queue::all_exit_ok(".join(", ",@_).")" if $trace;
   my @result=waitpids(@_);
   my $i;
-  for($i=0;$i<@result;$i++) {
+  for($i=0;$i<=$#result;$i++) {
     next unless $i&1;
     if (!defined($result[$i]) or $result[$i]) {
-      carp "Child ".$_[$i>>1]." fail with code $result[$i], waitpid return $result[$i-1]" if $debug;
+      carp "Child $_[$i>>1] fail with code $result[$i], waitpid return $result[$i-1]" if $debug;
       return 0;
     }
   }
@@ -381,25 +367,24 @@ sub running_now () {
 
 
 1;
-
 __END__
 
 # docs:
 
 =head1 NAME
 
-Proc::Queue - limit the number of child processes running
+Proc::Queue - Perl extension to limit the number of concurrent child
+process running
 
 =head1 SYNOPSIS
 
   use Proc::Queue size => 4, debug => 1;
 
   package other;
-  use POSIX ":sys_wait_h"; # imports WNOHANG
 
   # this loop creates new childs, but Proc::Queue makes it wait every
   # time the limit (4) is reached until enough childs exit
-  foreach (1..100) {
+  foreach (1..10) {
     my $f=fork;
     if(defined ($f) and $f==0) {
       print "-- I'm a forked process $$\n";
@@ -407,7 +392,6 @@ Proc::Queue - limit the number of child processes running
       print "-- I'm tired, going away $$\n";
       exit(0)
     }
-    1 while waitpid(-1, WNOHANG)>0; # reaps childs
   }
 
   Proc::Queue::size(10); # changing limit to 10 concurrent processes
@@ -416,7 +400,7 @@ Proc::Queue - limit the number of child processes running
   Proc::Queue::delay(0.2); # set 200 miliseconds as minimum
                            # delay between fork calls
 
-  package other; # just to test it works on any package
+  package other; # just to test it works in any package
 
   print "going again!\n";
 
@@ -436,51 +420,33 @@ Proc::Queue - limit the number of child processes running
 =head1 DESCRIPTION
 
 This module lets you parallelise a perl program using the C<fork>,
-C<exit>, C<wait> and C<waitpid> calls as usual but without taking care
-of creating too many processes and overloading the machine.
+C<exit>, C<wait> and C<waitpid> calls as usual and without the need to
+take care of creating too many processes that could overload the
+machine.
 
 It redefines perl C<fork>, C<exit>, C<wait> and C<waitpid> core
-functions. Old programs do not need to be reprogrammed, only the C<use
-Proc::Queue ...> sentence has to be added to them.
+functions. Old programs do not need to be modified, only the C<use
+Proc::Queue> sentence is needed.
 
-Additionally, the module has two debugging modes (debug and trace)
-that seem too be very useful when developing parallel aplications:
+Additionally, the module have two debugging modes (debug and trace)
+that seem too be very useful when developing parallel aplications.
 
-=over 4
+Debug mode when activated dumps lots of information about processes
+being created, exiting, being caught be parent, etc.
 
-=item debug mode:
+Trace mode just prints a line every time one of the C<fork>, C<exit>,
+C<wait> or C<waitpid> functions is called.
 
-when active, dumps lots of information about processes being created,
-exiting, being caught by parent, etc.
+It is also possible to set a minimun delay time between calls to fork
+to stop consecutive processes for starting in a short time interval.
 
-=item trace mode:
-
-prints a line every time one of the C<fork>, C<exit>, C<wait> or
-C<waitpid> functions are called.
-
-=back
-
-It is also possible to set a minimun delay time between fork calls
-to stop too many processes for starting in a short time interval.
-
-Child processes continue to use the modified functions, but their
-queues are reset and the maximun process number for them is set to 1
-(anyway, childs can change their queue size themselves).
+Child processes continue to use the modified functions, but its queues
+are reset and the maximun process number for them is set to 1 (childs
+can change its proccess queue size themselves later).
 
 Proc::Queue doesn't work if CHLD signal handler is set to
-C<IGNORE>.  
-
-Internally, Proc::Queue, automatically catches zombies and stores
-their exit status in a private hash. To avoid leaking memory in long
-running programs you have to call C<wait> or C<waitpid> to delete
-entries from that hash or alternatively active the C<ignore_childs>
-mode:
-
-  Proc::Queue::ignore_childs(1)
-
-or
-
-  use Proc::Queue ignore_childs=>1, ...
+C<IGNORE>. You have to call C<wait> or C<waitpid> to get rid of zombie
+processes even if you are not interested in their exit status.
 
 
 =head2 EXPORT
@@ -511,39 +477,21 @@ If no argument is given, the number of processes allowed is returned.
 
 =item delay(), delay($time)
 
-lets you set a minimun time in seconds to elapse between consecutive
-calls to fork. It is useful to avoid creating too many processes in
-a short time (that could degrade performance).
+delay lets you set a minimun time in seconds to elapse between every
+consecutive calls to fork. This is usefull for not creating to much
+processes in a very short time.
 
-If Time::HiRes module is available delays shorted that 1 second are
-allowed.
+If the Time::HiRes module is available it will be used and delays
+shorted that 1 second could be used.
 
 If no arg is given, the current delay is returned.
 
 To clear it use C<Proc::Queue::delay(0)>.
 
-=item ignore_childs($on)
-
-calling
-
-  Proc::Queue::ignore_childs(1);
-
-is the equivalent to
-
-  $SIG{CHLD}='IGNORE'
-
-when using Proc::Queue.
-
-=item debug(), debug($boolean), trace(), trace($boolean)
-
-Change or return the status for the debug and trace modes.
-
 =back
 
-
 Other utility subroutines that can be imported from Proc::Queue are:
 
-
 =over 4
 
 =item fork_now()
@@ -561,8 +509,8 @@ exportable so you can do...
 
 =item waitpids(@pid)
 
-waits for all the processes in @pid to exit. It returns an array
-with pairs of pid and exit values (pid1, exit1, pid2, exit2, pid3,
+Will wait for all the processes in @pid to exit. It returns an array
+with pairs pid and exit values (pid1, exit1, pid2, exit2, pid3,
 exit3,...) as returned by individual waitpid calls.
 
 =item run_back(\&code), run_back { code }
@@ -577,8 +525,8 @@ A mix between run_back and fork_now.
 =item system_back(@command)
 
 Similar to the C<system> call but runs the command in the background
-and waits for other childs to exit first if there are already too many
-running.
+and waits for some childs to exit first if there are already too many
+childs running.
 
 =item system_back_now(@command)
 
@@ -593,12 +541,16 @@ Do a C<waitpids> call and test that all the processes exit with code 0.
 
 Returns the number of child processes currently running.
 
+=item debug(), debug($boolean), trace(), trace($boolean)
+
+Change or return the status for the debug and trace modes.
+
 =item import(pkg,opt,val,opt,val,...,fnt_name,fnt_name,...)
 
 The import function is not usually explicitally called but by the
-C<use Proc::Queue> statement.
-
-Options allowed are C<size>, C<debug> and C<trace>, i.e:
+C<use Proc::Queue> statement. The options allowed are C<size>, C<debug>
+and C<trace> and they let you configure the module instead of using
+the C<size>, C<debug> or C<trace> module functions as in...
 
   use Proc::Queue size=>10, debug=>1;
 
@@ -610,10 +562,12 @@ function name to be imported.
 
 =head2 BUGS
 
-Proc::Queue is a very stable module, no bugs have been reported
-for a long time.
+Proc::Queue is a very stable module, and no bugs have been reported in
+a long time.
+
+Child (forking) behaviour althought deterministic could be changed to
+something better. I would accept any suggestions on it.
 
-Support for Win32 OSs is still experimental.
 
 =head1 SEE ALSO
 
@@ -624,7 +578,7 @@ contained in the module distribution.
 
 =head1 AUTHOR
 
-Salvador Fandio E<lt>sfandino@yahoo.comE<gt>
+Salvador Fandio <sfandino@yahoo.com>
 
 
 =head1 COPYRIGHT AND LICENSE
